#include "current.h"
#include "core.h"
#include "streaming.h"
#include "NU32.h"
/// @file current.c
/// @brief Implements the inner current control loop
/// @author Your Name
/// @version 3.2014
/// @date 2014-03-01

// The proportional and integral gains
// "static" means that these variables are visible only inside current.c
// thus, if you declare similar variables in motion.c as static,
// they can also be named kp and ki and there will be no issue
// (thus these variables are global within the module, but not throughout the 
// whole program)
static int kp = 0, ki = 0;

/// @brief Setup Timer 1, which runs the current control loop
/// @post  Timer 1 and its interrupt are enabled.  
///        Its frequency is 5 kHz and its interrupt priority should be 7
///// NOTE: static just means that this function is only visible within current.c
///	   code in other files cannot call this function
static void timer1_init(void);

// TODO: this is setup for interrupt priority of 1.
//       You will need to change this.
//       Remember that interrupt priority 7 uses the shadow register set (SRS)
void __ISR(_TIMER_1_VECTOR,IPL1SOFT) Current_Control_Interrupt(void)
{
	//TODO: invert E0 so we can see when the interrupt is triggered
	
	//the switch stament examines the core_state.
	//it then jumps to the appropriate case (so if core_state = PWM,
	//the switch statement will jump to the PWM case.)
	//every case should end with a break; statement.
	//If you do not end it with a break; statement, the code will fall through
	//to the subsequent case.
	// See core.h for more information about the states.
	//
	// We manage the states for you (via menu.c, take a look if you want)
	// (modifying core state is probably not a good idea as it will make
	// the state machine logic more complicated
	// our demo program only modifies core_state in menu.c 

	// NOTE: Your motion control interrupt will have a similar structure to this
	
	//TODO: when you are ready, remove this statement.
	//	it is only here to make the matlab menu function until
	//	you get your own code up and running.
	//	You will be calling this when core_state == TUNE to record
	//	the response of the current controller to the test reference
	//	(see below).
	//	
	streaming_record(7,18,21); // TODO: this statement to be removed whe you are ready

	switch (core_state)
	{
		case IDLE:
		{
			// TODO: put the H-Bridge in Idle (coast) mode
			break;
		}
		case PWM:
		{
			// TODO: set the pwm according to the pwm reference value
			break;
		}
		case TUNE:
		{	
			//TODO: 	
			// record the sensor, reference, and control effort
			// all you need to do is call the follwoing function here, and it will be streamed to the PC
			//  by code in streaming.c and menu.c.  
			//	streaming_record(reference signal , sensor reading, control effort);
			//
			//	you will also want to use streaming_record in your motion interrupt service routine
			//	when you are in the TRACK or HOLD state

			// in tune mode you are tracking a -200mA to 200mA square wave at 100 Hz.
			break;
		}
		case TRACK:
		{
			//TODO:
			// in tracking mode the motion.c code sets a current reference
			//using current_amps_set().  THerefore, here, we should make sure
			//that the motion control loop gets the current it requests
			break;
		}
		case HOLD:
		{
			//TODO:
			// make sure that the motion control loop gets the current
			// it has requested via current_amps_set()
			break;
		}
		default:	
		{	
			// NOTE: bug free code should never get here.
			// its good practice to handle this
			// case and do something safe like turn the motors off
			// as reaching here indicates a bug in the code
			// this is an invalid state, stop the motors
			break;
		}
	}

	IFS0bits.T1IF = 0; // clear the interrupt flag
}

void current_init(void)
{
	//TODO: Uh Oh, somebody made the timer run too fast and at the
	//wrong priority.  FIXME
	// setup the current loop timer for for 5 kHz operation
	timer1_init();

	//TODO: setup the appropriate output compare pins and a timer for
	// 20 kHz PWM operation

	//TODO: setup pin e0 for digital I/O. this is just so that we can
	//verify the control loop frequency on the nscope.
	//NOTE: due to a bug in the nscope firmware, the frequency displayed
	//	may not exactly match what you specify here, but it should be close

	//We register the gains. This allows core.c to handle saving them
	core_register_int(&kp);
	core_register_int(&ki);

}


void current_pwm_set(int duty_percent)
{
	//TODO:
	//this is called by the menu when the user wants to specify
	//pwm duty cycles directly.
	
	// convert the duty_percent into a duty cycle
	// and assign it to a variable that stores the pwm reference.
	// Your interrupt will set this pwm mode when in the PWM state
}

void current_amps_set(int amps)
{
	//TODO:
	// set the amp reference
	// saturate at +/- 2000 mA
}

short current_amps_get()
{
	// TODO: read from the ADC and convert
	// the tics into mA
	return 0xBEEF;
}


void current_gains_sprintf(char * buffer)
{
	// TODO: sprintf the gains ki and kp into the buffer
	//For Now we return return a dummy buffer with fixed gains
	//you will need to remove this
	buffer[0]='1'; //remove me when implemented
	buffer[1]=' '; //remove me when implemented
	buffer[2]='7'; //remove me when implemented
	buffer[3]='\0';//remove me when implemented
}

void current_gains_sscanf(const char * buffer)
{
	//TODO: buffer will contain the gains as two numbers separated by 
	// a space.  scanf them from buffer and store the gains in
	// kp and ki

}

static void timer1_init(void)
{
	// setup timer 1
	// TODO: Make the timer run at 5kHz
	// whoever coded this is fired!
	T1CONbits.TCS = 0;    	  
	T1CONbits.TCKPS = 0b00;   
	PR1 = 30000; 	   	
	TMR1 = 0;   	  
	IPC1bits.T1IP = 1; 
	IPC1bits.T1IS = 0;
	IFS0bits.T1IF = 0;
	IEC0bits.T1IE = 1;
	T1CONbits.ON  = 1;

}


